try:
    import numpy as N
    import numpy.random as RandomArray
    haveNumpy = True
except ImportError:
            haveNumpy = False
            errorText = (
            "The FloatCanvas requires the numpy module, version 1.* \n\n"
            "You can get info about it at:\n"
            "http://numpy.scipy.org/\n\n"
            )
import wx

def DrawFrame():
    try:
        from floatcanvas import NavCanvas, FloatCanvas, Resources
    except ImportError:
        from wx.lib.floatcanvas import NavCanvas, FloatCanvas, Resources, GUIMode

    class DragShape:
        def __init__(self, bmp):
            self.bmp = bmp
            self.pos = (0,0)
            self.shown = True
            self.text = None
            self.fullscreen = False

        def HitTest(self, pt):
            rect = self.GetRect()
            return rect.InsideXY(pt.x, pt.y)

        def GetRect(self):
            return wx.Rect(self.pos[0], self.pos[1],
                          self.bmp.GetWidth(), self.bmp.GetHeight())

        def Draw(self, dc, op = wx.COPY):
            if self.bmp.Ok():
                memDC = wx.MemoryDC()
                memDC.SelectObject(self.bmp)

                dc.Blit(self.pos[0], self.pos[1],
                        self.bmp.GetWidth(), self.bmp.GetHeight(),
                        memDC, 0, 0, op, True)

                return True
            else:
                return False

    class Canvas(wx.Panel):
        def __init__(self, parent, id, size = wx.DefaultSize, **kwargs):
            wx.Panel.__init__(self, parent, id, size=(500,500))

            self.shapes = []
            self.dragImage = None
            self.dragShape = None
            self.hiliteShape = None

            bmp = wx.Bitmap('image/ttest.bmp')
            shape = DragShape(bmp)
            shape.pos = (1, 1)
            shape.fullscreen = False
            self.shapes.append(shape)
            
            self.Bind(wx.EVT_ERASE_BACKGROUND, lambda x:None)
            self.Bind(wx.EVT_PAINT, self.OnPaint)
            self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
            self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
            self.Bind(wx.EVT_MOTION, self.OnMotion)

            self.dragid = -1
            self.lastpos = (0,0)

            return None

        def SetProjectionFun(self,ProjectionFun):
            if ProjectionFun == 'FlatEarth':
                self.ProjectionFun = self.FlatEarthProjection
            elif callable(ProjectionFun):
                self.ProjectionFun = ProjectionFun
            elif ProjectionFun is None:
                self.ProjectionFun = lambda x=None: N.array( (1,1), N.float)
            else:
                raise FloatCanvasError('Projectionfun must be either:'
                                       ' "FlatEarth", None, or a callable object '
                                       '(function, for instance) that takes the '
                                       'ViewPortCenter and returns a MapProjectionVector')

        def FlatEarthProjection(self, CenterPoint):
            MaxLatitude = 75
            MinLatitude = -75
            Lat = min(CenterPoint[1],MaxLatitude)
            Lat = max(Lat,MinLatitude)
            return N.array((N.cos(N.pi*Lat/180),1),N.float)

        def OnPaint(self, event):
            dc = wx.PaintDC(self)
            self.PrepareDC(dc)
            self.DrawShapes(dc)


        def DrawShapes(self, dc):
            for shape in self.shapes:
                if shape.shown:
                    shape.Draw(dc)

        def FindShape(self, pt):
            for shape in self.shapes:
                if shape.HitTest(pt):
                    return shape
            return None

        def OnLeftDown(self, event):
            shape = self.FindShape(event.GetPosition())
            if shape:
                self.dragShape = shape
                self.dragStartPos = event.GetPosition()
            print "down"
            
        def OnLeftUp(self, event):
            if not self.dragImage or not self.dragShape:
                self.dragImage = None
                self.dragShape = None
                return

            self.dragImage.Hide()
            self.dragImage.EndDrag()
            self.dragImage = None

            if self.hiliteShape:
                self.RefreshRect(self.hiliteShape.GetRect())
                self.hiliteShape = None

            self.dragShape.pos = (
                self.dragShape.pos[0] + event.GetPosition()[0] - self.dragStartPos[0],
                self.dragShape.pos[1] + event.GetPosition()[1] - self.dragStartPos[1]
                )
                
            self.dragShape.shown = True
            self.RefreshRect(self.dragShape.GetRect())
            self.dragShape = None

        def OnMotion(self, event):
            if not self.dragShape or not event.Dragging() or not event.LeftIsDown():
                return

            if self.dragShape and not self.dragImage:

                tolerance = 2
                pt = event.GetPosition()
                dx = abs(pt.x - self.dragStartPos.x)
                dy = abs(pt.y - self.dragStartPos.y)
                if dx <= tolerance and dy <= tolerance:
                    return
                self.dragShape.shown = False
                self.RefreshRect(self.dragShape.GetRect(), True)
                self.Update()

                if self.dragShape.text:
                    self.dragImage = wx.DragString(self.dragShape.text,
                                                  wx.StockCursor(wx.CURSOR_HAND))
                else:
                    self.dragImage = wx.DragImage(self.dragShape.bmp,
                                                 wx.StockCursor(wx.CURSOR_HAND))

                hotspot = self.dragStartPos - self.dragShape.pos
                self.dragImage.BeginDrag(hotspot, self, self.dragShape.fullscreen)

                self.dragImage.Move(pt)
                self.dragImage.Show()


            elif self.dragShape and self.dragImage:
                onShape = self.FindShape(event.GetPosition())
                unhiliteOld = False
                hiliteNew = False

                if self.hiliteShape:
                    if onShape is None or self.hiliteShape is not onShape:
                        unhiliteOld = True

                if onShape and onShape is not self.hiliteShape and onShape.shown:
                    hiliteNew = True

                if unhiliteOld or hiliteNew:
                    self.dragImage.Hide()

                if unhiliteOld:
                    dc = wx.ClientDC(self)
                    self.hiliteShape.Draw(dc)
                    self.hiliteShape = None

                if hiliteNew:
                    dc = wx.ClientDC(self)
                    self.hiliteShape = onShape
                    self.hiliteShape.Draw(dc, wx.INVERT)

                self.dragImage.Move(event.GetPosition())
                if unhiliteOld or hiliteNew:
                    self.dragImage.Show()



    class MainFrame(wx.Frame):
        def __init__(self, parent):
            wx.Frame.__init__(self, parent, -1, "Test", size=(800,600))
            self.SetBackgroundColour('White gray')

            self.panel_1 = wx.Panel(self, -1, style=wx.SUNKEN_BORDER, size=(500, 500))
            self.panel_2 = wx.Panel(self, -1)
            self.panel_3 = wx.Panel(self.panel_2, -1, size=(300, 300), pos=(500, 0))
            self.panel_4 = wx.Panel(self.panel_2, -1, style=wx.RAISED_BORDER, size=(280, 280), pos=(500, 300))

            self.Canvas = Canvas(self.panel_1, wx.ID_ANY)

            ## Add the message window on panel_1
            self.MsgWindow = wx.TextCtrl(self.panel_1, wx.ID_ANY,
                                         "Event Message : ",
                                         style = (wx.TE_MULTILINE |
                                                  wx.TE_READONLY |
                                                  wx.SUNKEN_BORDER)
                                         )

            ## make TreeCtrl on panel_3
            self.txt_Tree = wx.StaticText(self.panel_3, -1, "Images")
            self.dir_Tree = wx.TreeCtrl(self.panel_3, -1, pos=(0, 20), size=(205, 270))
            self.root = self.dir_Tree.AddRoot("Image")
            
            ## Create a sizer to manage panels
            v_box = wx.BoxSizer(wx.VERTICAL)
            v_box.Add(self.panel_3, 1, flag = wx.EXPAND)
            v_box.Add((5,5))
            v_box.Add(self.panel_4, 0, flag = wx.EXPAND)
            v_box.Add((5,5))
            self.panel_2.SetSizer(v_box)

            h_box = wx.BoxSizer(wx.HORIZONTAL)
            h_box.Add(self.panel_1, 1, flag = wx.EXPAND)
            h_box.Add((5,5))
            h_box.Add(self.panel_2, 0, flag = wx.EXPAND)
            h_box.Add((5,5))
            self.SetSizer(h_box)

            ## Create a sizer to manage the Canvas
            main_box = wx.BoxSizer(wx.VERTICAL)
            main_box.Add(self.Canvas, 4, flag = wx.GROW)
            main_box.Add(self.MsgWindow, 1, wx.EXPAND | wx.ALL, 5)
            self.panel_1.SetSizer(main_box)


            self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
            return None

        def OnCloseWindow(self, event):
            self.Destroy()
        def OnQuit(self, event):
            self.Close(True)

        def DrawMap(self,event = None):
            wx.GetApp().Yield(True)
            import os, time
            
            self.Canvas.SetProjectionFun("FlatEarth")
            wx.GetApp().Yield(True)
            Shorelines = self.Read_MapGen(os.path.join("data",'world.dat'),stats = 0)
            for segment in Shorelines:
                self.Canvas.AddLine(segment)

        def Read_MapGen(self, filename, stats = 0,AllLines=0):
            import string
            file = open(filename,'rt')
            data = file.readlines()
            data = map(string.strip,data)

            Shorelines = []
            segment = []
            for line in data:
                if line:
                    if line == "# -b":
                        if segment: Shorelines.append(N.array(segment))
                        segment = []
                    else:
                        segment.append(map(float,string.split(line)))
            if segment: Shorelines.append(N.array(segment))

            if stats:
                NumSegments = len(Shorelines)
                NumPoints = 0
                for segment in Shorelines:
                    NumPoints = NumPoints + len(segment)
                AvgPoints = NumPoints / NumSegments
            if AllLines:
                Lines = []
                for segment in Shorelines:
                    Lines.append(segment[0])
                    for point in segment[1:-1]:
                        Lines.append(point)
                        Lines.append(point)
                    Lines.append(segment[-1])
                return Lines
            else:
                return Shorelines


    return MainFrame


if __name__ == "__main__":
    import wx
    import os, sys, getopt, cPickle, cStringIO
    app = wx.PySimpleApp()
    MainFrame = DrawFrame()
    frame = MainFrame(None)
    frame.Show()
    frame.DrawMap()
    app.MainLoop()


try:
    import numpy as N
    import numpy.random as RandomArray
    haveNumpy = True
except ImportError:
            haveNumpy = False
            errorText = (
            "The FloatCanvas requires the numpy module, version 1.* \n\n"
            "You can get info about it at:\n"
            "http://numpy.scipy.org/\n\n"
            )
import wx

def DrawFrame():
    try:
        from floatcanvas import NavCanvas, FloatCanvas, Resources
    except ImportError:
        from wx.lib.floatcanvas import NavCanvas, FloatCanvas, Resources, GUIMode

    class DragShape:
        def __init__(self, bmp):
            self.bmp = bmp
            self.pos = (0,0)
            self.shown = True
            self.text = None
            self.fullscreen = False

        def HitTest(self, pt):
            rect = self.GetRect()
            return rect.InsideXY(pt.x, pt.y)

        def GetRect(self):
            return wx.Rect(self.pos[0], self.pos[1],
                          self.bmp.GetWidth(), self.bmp.GetHeight())

        def Draw(self, dc, op = wx.COPY):
            if self.bmp.Ok():
                memDC = wx.MemoryDC()
                memDC.SelectObject(self.bmp)

                dc.Blit(self.pos[0], self.pos[1],
                        self.bmp.GetWidth(), self.bmp.GetHeight(),
                        memDC, 0, 0, op, True)

                return True
            else:
                return False

    class Canvas(wx.Panel):
        def __init__(self, parent, id, size = wx.DefaultSize, **kwargs):
            wx.Panel.__init__(self, parent, id, size=(500,500))

            self.shapes = []
            self.dragImage = None
            self.dragShape = None
            self.hiliteShape = None

            bmp = wx.Bitmap('image/ttest.bmp')
            shape = DragShape(bmp)
            shape.pos = (1, 1)
            shape.fullscreen = False
            self.shapes.append(shape)
            
            self.Bind(wx.EVT_ERASE_BACKGROUND, lambda x:None)
            self.Bind(wx.EVT_PAINT, self.OnPaint)
            self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
            self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
            self.Bind(wx.EVT_MOTION, self.OnMotion)

            self.dragid = -1
            self.lastpos = (0,0)

            return None

        def SetProjectionFun(self,ProjectionFun):
            if ProjectionFun == 'FlatEarth':
                self.ProjectionFun = self.FlatEarthProjection
            elif callable(ProjectionFun):
                self.ProjectionFun = ProjectionFun
            elif ProjectionFun is None:
                self.ProjectionFun = lambda x=None: N.array( (1,1), N.float)
            else:
                raise FloatCanvasError('Projectionfun must be either:'
                                       ' "FlatEarth", None, or a callable object '
                                       '(function, for instance) that takes the '
                                       'ViewPortCenter and returns a MapProjectionVector')

        def FlatEarthProjection(self, CenterPoint):
            MaxLatitude = 75
            MinLatitude = -75
            Lat = min(CenterPoint[1],MaxLatitude)
            Lat = max(Lat,MinLatitude)
            return N.array((N.cos(N.pi*Lat/180),1),N.float)

        def OnPaint(self, event):
            dc = wx.PaintDC(self)
            self.PrepareDC(dc)
            self.DrawShapes(dc)


        def DrawShapes(self, dc):
            for shape in self.shapes:
                if shape.shown:
                    shape.Draw(dc)

        def FindShape(self, pt):
            for shape in self.shapes:
                if shape.HitTest(pt):
                    return shape
            return None

        def OnLeftDown(self, event):
            shape = self.FindShape(event.GetPosition())
            if shape:
                self.dragShape = shape
                self.dragStartPos = event.GetPosition()
            print "down"
            
        def OnLeftUp(self, event):
            if not self.dragImage or not self.dragShape:
                self.dragImage = None
                self.dragShape = None
                return

            self.dragImage.Hide()
            self.dragImage.EndDrag()
            self.dragImage = None

            if self.hiliteShape:
                self.RefreshRect(self.hiliteShape.GetRect())
                self.hiliteShape = None

            self.dragShape.pos = (
                self.dragShape.pos[0] + event.GetPosition()[0] - self.dragStartPos[0],
                self.dragShape.pos[1] + event.GetPosition()[1] - self.dragStartPos[1]
                )
                
            self.dragShape.shown = True
            self.RefreshRect(self.dragShape.GetRect())
            self.dragShape = None

        def OnMotion(self, event):
            if not self.dragShape or not event.Dragging() or not event.LeftIsDown():
                return

            if self.dragShape and not self.dragImage:

                tolerance = 2
                pt = event.GetPosition()
                dx = abs(pt.x - self.dragStartPos.x)
                dy = abs(pt.y - self.dragStartPos.y)
                if dx <= tolerance and dy <= tolerance:
                    return
                self.dragShape.shown = False
                self.RefreshRect(self.dragShape.GetRect(), True)
                self.Update()

                if self.dragShape.text:
                    self.dragImage = wx.DragString(self.dragShape.text,
                                                  wx.StockCursor(wx.CURSOR_HAND))
                else:
                    self.dragImage = wx.DragImage(self.dragShape.bmp,
                                                 wx.StockCursor(wx.CURSOR_HAND))

                hotspot = self.dragStartPos - self.dragShape.pos
                self.dragImage.BeginDrag(hotspot, self, self.dragShape.fullscreen)

                self.dragImage.Move(pt)
                self.dragImage.Show()


            elif self.dragShape and self.dragImage:
                onShape = self.FindShape(event.GetPosition())
                unhiliteOld = False
                hiliteNew = False

                if self.hiliteShape:
                    if onShape is None or self.hiliteShape is not onShape:
                        unhiliteOld = True

                if onShape and onShape is not self.hiliteShape and onShape.shown:
                    hiliteNew = True

                if unhiliteOld or hiliteNew:
                    self.dragImage.Hide()

                if unhiliteOld:
                    dc = wx.ClientDC(self)
                    self.hiliteShape.Draw(dc)
                    self.hiliteShape = None

                if hiliteNew:
                    dc = wx.ClientDC(self)
                    self.hiliteShape = onShape
                    self.hiliteShape.Draw(dc, wx.INVERT)

                self.dragImage.Move(event.GetPosition())
                if unhiliteOld or hiliteNew:
                    self.dragImage.Show()



    class MainFrame(wx.Frame):
        def __init__(self, parent):
            wx.Frame.__init__(self, parent, -1, "Test", size=(800,600))
            self.SetBackgroundColour('White gray')

            self.panel_1 = wx.Panel(self, -1, style=wx.SUNKEN_BORDER, size=(500, 500))
            self.panel_2 = wx.Panel(self, -1)
            self.panel_3 = wx.Panel(self.panel_2, -1, size=(300, 300), pos=(500, 0))
            self.panel_4 = wx.Panel(self.panel_2, -1, style=wx.RAISED_BORDER, size=(280, 280), pos=(500, 300))

            self.Canvas = Canvas(self.panel_1, wx.ID_ANY)

            ## Add the message window on panel_1
            self.MsgWindow = wx.TextCtrl(self.panel_1, wx.ID_ANY,
                                         "Event Message : ",
                                         style = (wx.TE_MULTILINE |
                                                  wx.TE_READONLY |
                                                  wx.SUNKEN_BORDER)
                                         )

            ## make TreeCtrl on panel_3
            self.txt_Tree = wx.StaticText(self.panel_3, -1, "Images")
            self.dir_Tree = wx.TreeCtrl(self.panel_3, -1, pos=(0, 20), size=(205, 270))
            self.root = self.dir_Tree.AddRoot("Image")
            
            ## Create a sizer to manage panels
            v_box = wx.BoxSizer(wx.VERTICAL)
            v_box.Add(self.panel_3, 1, flag = wx.EXPAND)
            v_box.Add((5,5))
            v_box.Add(self.panel_4, 0, flag = wx.EXPAND)
            v_box.Add((5,5))
            self.panel_2.SetSizer(v_box)

            h_box = wx.BoxSizer(wx.HORIZONTAL)
            h_box.Add(self.panel_1, 1, flag = wx.EXPAND)
            h_box.Add((5,5))
            h_box.Add(self.panel_2, 0, flag = wx.EXPAND)
            h_box.Add((5,5))
            self.SetSizer(h_box)

            ## Create a sizer to manage the Canvas
            main_box = wx.BoxSizer(wx.VERTICAL)
            main_box.Add(self.Canvas, 4, flag = wx.GROW)
            main_box.Add(self.MsgWindow, 1, wx.EXPAND | wx.ALL, 5)
            self.panel_1.SetSizer(main_box)


            self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
            return None

        def OnCloseWindow(self, event):
            self.Destroy()
        def OnQuit(self, event):
            self.Close(True)

        def DrawMap(self,event = None):
            wx.GetApp().Yield(True)
            import os, time
            
            self.Canvas.SetProjectionFun("FlatEarth")
            wx.GetApp().Yield(True)
            Shorelines = self.Read_MapGen(os.path.join("data",'world.dat'),stats = 0)
            for segment in Shorelines:
                self.Canvas.AddLine(segment)

        def Read_MapGen(self, filename, stats = 0,AllLines=0):
            import string
            file = open(filename,'rt')
            data = file.readlines()
            data = map(string.strip,data)

            Shorelines = []
            segment = []
            for line in data:
                if line:
                    if line == "# -b":
                        if segment: Shorelines.append(N.array(segment))
                        segment = []
                    else:
                        segment.append(map(float,string.split(line)))
            if segment: Shorelines.append(N.array(segment))

            if stats:
                NumSegments = len(Shorelines)
                NumPoints = 0
                for segment in Shorelines:
                    NumPoints = NumPoints + len(segment)
                AvgPoints = NumPoints / NumSegments
            if AllLines:
                Lines = []
                for segment in Shorelines:
                    Lines.append(segment[0])
                    for point in segment[1:-1]:
                        Lines.append(point)
                        Lines.append(point)
                    Lines.append(segment[-1])
                return Lines
            else:
                return Shorelines


    return MainFrame


if __name__ == "__main__":
    import wx
    import os, sys, getopt, cPickle, cStringIO
    app = wx.PySimpleApp()
    MainFrame = DrawFrame()
    frame = MainFrame(None)
    frame.Show()
    frame.DrawMap()
    app.MainLoop()
